/**
 * @fileoverview Implementation of spritesheet component. 
 * @author pvoelcker@google.com (Patrick Voelcker)
 */
if (document.registerElement) {

    var dispatchEvent = function (eventName, source) {
        var event = document.createEvent('Event');
        event.initEvent(eventName, true, true);
        source.dispatchEvent(event);
        return event;
    };

    var proto = Object.create(HTMLElement.prototype, {

        createdCallback: {
            value: function () {
                this.canvas = null;
            },
            enumerable: true
        },

        attachedCallback: {
            value:
            /**
             * Lifecycle callback that is invoked when this element is added to the
             * DOM.
             */
                function () {
                this.generateSprite();
            },
            enumerable: true
        },

        detachedCallback: {
            value:
            /**
             * Lifecycle callback that is invoked when this element is removed from
             * the DOM.
             */
                function () {},
            enumerable: true
        },

        attributeChangedCallback: {
            value:
            /**
             * Lifecycle callback that is invoked when an attribute is changed.
             * @param {string} attributeName Name of the attribute being changed.
             */
                function (attributeName) {

                if (!this.canvas) {
                    // It is possible that the attribute is set before the
                    // component is added to the DOM.
                    return;
                }
                switch (attributeName) {
                    case 'image':
                        this.generateSprite();
                        break;
                }
            },
            enumerable: true
        },


        generateSprite: {
            value: function () {

                if (!this.canvas) {

                    // getting component size
                    this.componentWidth = parseInt(window.getComputedStyle(this, null).getPropertyValue("width"));
                    this.componentHeight = parseInt(window.getComputedStyle(this, null).getPropertyValue("height"));

                    console.log(this.componentWidth + "=" + parseInt(window.getComputedStyle(this, null).getPropertyValue("width")));

                    // getting parameters from component options

                    // getting url of sprite sheet
                    this.imageUrl = this.getAttribute('image');

                    // getting x- and y-position, width and height of selected sprite in sprite sheet
                    this.sX = parseInt(this.getAttribute('sX') || 0);
                    this.sY = parseInt(this.getAttribute('sY') || 0);
                    this.sWidth = parseInt(this.getAttribute('sWidth') || 64);
                    this.sHeight = parseInt(this.getAttribute('sHeight') || 64);

                    // getting scale mode
                    this.sScale = this.getAttribute('sScale');

                    // generate canvas within component on stage
                    this.canvas = document.createElement('canvas');

					// set size of canvas
                    // setting scale factor, according to sScale from component options 
                    if ((this.sScale === '') || (this.sScale === 'original size')) {
                        // original size
                        this.canvas.style.width = this.sWidth + 'px';
                        this.canvas.style.height = this.sHeight + 'px';
                    } else if (this.sScale === 'stretch to component') {
                        // stretch to exact component size
                        this.canvas.style.width = "100%";
                        this.canvas.style.height = "100%";
                    } else if (this.sScale === 'fit to component') {
                    	var ratioW = this.componentWidth / this.sWidth; 
            			var ratioH = this.componentHeight / this.sHeight;
            			
            			// smaller ratio will ensure that the image fits in the view
            			var ratio = ratioW < ratioH ? ratioW : ratioH;
            			
            			this.canvas.style.width = (this.sWidth * ratio) + "px";
            			this.canvas.style.height = (this.sHeight * ratio) + "px";
                    }
                    this.canvas.width = this.sWidth;
                    this.canvas.height = this.sHeight;
                    
                    // get context of canvas
                    this.context = this.canvas.getContext('2d');

                    // add canvas to component area
                    this.appendChild(this.canvas);

                    // load source image
                    this.sourceImage = new Image();

                    // after loading, copy image clip to canvas
                    this.sourceImage.addEventListener("load", function (e) {
                        // get context of canvas
                        var context = this.canvas.getContext('2d');
                        // copy image clip to canvas
                        context.drawImage(this.sourceImage, this.sX, this.sY,
                                          this.sWidth, this.sHeight,
                                          0, 0,
                                          this.sWidth, this.sHeight);
                      	this.sourceImage = null;
                    }.bind(this));

                    // assign image url to image object after load-handler to avoid caching issues
                    this.sourceImage.src = this.imageUrl;
                }

            }
        }


    });
    document.registerElement('img-spritesheet', {
        prototype: proto
    });
}